//FRAGMENT SHADER

// inputs
uniform lowp vec4 blendColor;
uniform lowp vec4 noBlackBlendColor;

uniform highp vec4 pixelToPhoto;

uniform lowp float blurIntensity;
uniform lowp float additiveIntensity;
uniform lowp float contrastIntensity;
uniform lowp float brightnessIntensity;
uniform lowp float differenceIntensity;
uniform lowp float saturationIntensity;



// general variables for laying out blend
uniform sampler2D image_texture; // the image texture object

varying highp vec2 v_baseTextureCoordinate; // texture coordinates for the image
uniform sampler2D glow_texture; // the image texture object

varying highp vec2 v_textureCoordinate; // texture coordinates for the image
varying lowp vec2 v_redTextureCoordinate; // texture coordinates for red

#ifdef USES_BLUR
varying lowp vec2 v_blurTexCoords[4];
varying lowp float bias;
#endif

//varying lowp float localBlurIntensity;
varying lowp float localContrastIntensity;
varying lowp float localBrightnessIntensity;
varying lowp float localAdditiveIntensity;
varying lowp float localDifferenceIntensity;
varying lowp float localSaturationIntensity;

// gausian filter weights creating using: http://www.embege.com/gauss/

void main() {
    
    mediump vec4 imageTexelColor;
    
    
    // apply the blur
    
#ifdef USES_BLUR
    
    // center pixel
    imageTexelColor = texture2D(image_texture, v_textureCoordinate, bias)*0.20417995557165622;
    
    
    // above and below
    imageTexelColor += texture2D(image_texture, v_blurTexCoords[ 0], bias)*0.1989550111;
    imageTexelColor += texture2D(image_texture, v_blurTexCoords[ 1], bias)*0.1989550111;
    imageTexelColor += texture2D(image_texture, v_blurTexCoords[ 2], bias)*0.1989550111;
    imageTexelColor += texture2D(image_texture, v_blurTexCoords[ 3], bias)*0.1989550111;
    
    #ifdef USES_REDBLUE
        
        // use the bias if we're blurring to give it a little blur
        imageTexelColor.r = texture2D(image_texture, v_redTextureCoordinate, bias).r;
        
    #endif
    
    
#else
    
    imageTexelColor =  texture2D(image_texture, v_textureCoordinate);
    
    #ifdef USES_REDBLUE
        
        imageTexelColor.r = texture2D(image_texture, v_redTextureCoordinate).r;
        
    #endif
    
#endif

    
#ifdef USES_DIFFERENCE
    // apply the difference
    imageTexelColor = abs(imageTexelColor - (noBlackBlendColor * localDifferenceIntensity));
#endif

#ifdef USES_BRIGHTNESS
    // apply the brightness
    imageTexelColor = imageTexelColor + (localBrightnessIntensity * noBlackBlendColor);

#endif
   
#ifdef USES_CONTRAST
    // apply the contrast
    imageTexelColor = (imageTexelColor - 0.5) * (1.0 + (localContrastIntensity * 3.0)) + 0.5;
    imageTexelColor = imageTexelColor * (1.0 - localContrastIntensity) + (imageTexelColor * blendColor) * localContrastIntensity;

#endif
    
#ifdef USES_ADDITIVE
    // apply the additive blend
    highp vec4 blendedTexelColor = imageTexelColor + (imageTexelColor * localAdditiveIntensity * 2.0 * noBlackBlendColor);
    imageTexelColor = (imageTexelColor * (1.0 - localAdditiveIntensity)) + (blendedTexelColor * localAdditiveIntensity);
#endif
    
#ifdef USES_SATURATION
    // apply the saturation blend
    highp vec3 grayXfer = vec3(0.3, 0.59, 0.11);
	highp vec4 gray = vec4(dot(grayXfer, imageTexelColor.rgb));
    imageTexelColor = (imageTexelColor * (1.0 - localSaturationIntensity)) + (gray * localSaturationIntensity);
#endif
    
    imageTexelColor.a = 1.0;
    

    
// don't draw glow if we're drawing photo only
#if defined(USES_GLOW) && !defined(IS_DRAWING_PHOTO_ONLY)

    lowp vec4 glowColor = texture2D(glow_texture, v_baseTextureCoordinate);

    #ifdef INVERT_GLOW
        imageTexelColor.rgb = mix(imageTexelColor.rgb, glowColor.rgb, 1.0-glowColor.a);
    #else
          imageTexelColor.rgb = mix(imageTexelColor.rgb, glowColor.rgb, glowColor.a);
    #endif
        

    // if we're drawing the fragment only then just apply glow to a fully transparent color
    #if defined(IS_PHOTO_PASS) && defined(IS_DRAWING_TOP_LAYER_ONLY)
        imageTexelColor = glowColor;
        imageTexelColor.rgb = imageTexelColor.rgb * imageTexelColor.a;
    #endif
    
#endif
    
    gl_FragColor = imageTexelColor;
}



