//VERTEX SHADER

// inputs
attribute vec4 vertexAttribPosition;
attribute vec2 vertexAttribOuterOutlineNormal;

uniform highp mat4 modelScaleTransform;
uniform highp mat4 modelViewTransform;
uniform highp mat3 normalTransform;
uniform highp mat4 projectionViewTransform;

uniform lowp float outlineIntensity;
varying lowp float outlineAlpha;

uniform lowp float maxInnerOutline;




void main()
{
    // POSITION AND LOCAL INTENSITY
    /////////////////////////////////////////////////////////////////

    // pass the texture coords on to the fragment shader as a varying attribute

    vec4 position = modelScaleTransform * vertexAttribPosition;

#ifdef OUTLINE_NEGATIVE

    lowp float offset = clamp(abs(outlineIntensity), 0.0, 0.3); // clamp this so the outlines only get to a certain width
    outlineAlpha = -outlineIntensity;

#else
    lowp float offset = clamp(outlineIntensity, -2.0, 2.0); // clamp this so the outlines only get to a certain width
    outlineAlpha = 1.0;
#endif

    offset = offset * 0.03;

    // if we've hit the max for the inner outline grow on the outside instead
    offset = offset + max(offset - maxInnerOutline, 0.0);

    position.xz = position.xz - (offset * vertexAttribOuterOutlineNormal);

    position = modelViewTransform * position;
    // set the position of the vertex (no transform, these are flat)



    gl_Position = projectionViewTransform * position;

}

